import {
  getInitialPoint
} from './annotation/utils';
class InkSign {
  constructor({ container, pdfViewer, eventBus }) {
    this.container = container
    this.pdfViewer = pdfViewer
    this.eventBus = eventBus

    this.path = []

    this.onMousedown = this.handleMouseDown.bind(this)
    this.onMousemove = this.handleMouseMove.bind(this)
    this.onMouseup = this.handleMouseUp.bind(this)
    this.onHandleGetSignData = this.getData.bind(this)
    this.onHandleInitCursorStyle = this.initCursorStyle.bind(this)
    this.init()
  }

  init () {
    this.canvas = document.getElementById('trackpadCanvas')
    this.ctx = this.canvas.getContext('2d')
    this.canvas.style.cursor = 'crosshair'

    this.canvasWidth = this.canvas.width
    this.canvasHeight = this.canvas.height

    this.canvas.addEventListener('mousedown', this.onMousedown)

    this.eventBus._on('getSignData', this.onHandleGetSignData)
    this.eventBus._on('initCursorStyle', this.onHandleInitCursorStyle)
  }

  handleMouseDown (e) {
    //计算鼠标在画布的距离
    var disX = e.clientX - this.canvas.offsetLeft
    var disY = e.clientY - this.canvas.offsetTop
    //设置画线的宽，与颜色
    this.ctx.lineWidth = 3;
    this.ctx.strokeStyle = 'black';
    this.ctx.lineJoin = 'round';
    this.ctx.lineCap = 'round';

    this.ctx.beginPath();
    //设置画的起始点
    this.ctx.moveTo(disX, disY)

    document.addEventListener('mousemove', this.onMousemove)
    document.addEventListener('mouseup', this.onMouseup)
  }
  
  handleMouseMove (e) {
    var disX = e.clientX - this.canvas.offsetLeft
    var disY = e.clientY - this.canvas.offsetTop
    if (!(disX > this.canvasWidth || disY > this.canvasHeight || disX < 0 || disY < 0)) {
      //移动时设置画线的结束位置。并且显示
      this.ctx.lineTo(disX, disY) //鼠标点下去的位置
      this.ctx.stroke()
      this.path.push([disX, disY])
    }
  }

  handleMouseUp () {
    document.removeEventListener('mousemove', this.onMousemove)
    document.removeEventListener('mouseup', this.onMouseup)

    const rect = this.getRect()
    this.rect = rect
  }

  clear () {
    this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height)
    this.path = []
    this.initCursorStyle()
  }

  reset () {
    this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height)
    this.path = []
    this.onMousedown = null
    this.onMousemove = null
    this.onMouseup = null
    this.onHandleGetSignData = null
    this.onHandleInitCursorStyle = null
  }

  getRect () {
    const xArray = [];
    const yArray = [];
    this.path.forEach((point) => {
      xArray.push(point[0]);
      yArray.push(point[1]);
    });
    const top = Math.min(...yArray);
    const left = Math.min(...xArray);
    const bottom = Math.max(...yArray);
    const right = Math.max(...xArray);
    // this.addPath = this.addPath.map((ele) => {
    //   let x = ele[0] - left
    //   let y = ele[1] - top
    //   return [x, y]
    // })
    return {
      left,
      top,
      bottom,
      right,
      width: right - left,
      height: bottom - top
    }
  }

  toImage () {
    if (this.path.length === 0) return
    const finalCanvas = this.paintFinalCanvas()
    const dataURL = finalCanvas.toDataURL('image/jpeg')
    this.dataURL = dataURL

    // 创建一个Image元素
    var img = new Image();
    img.src = dataURL;
    img.id = 'sign-image-save'

    // 设置样式和光标
    img.style.position = "fixed";
    img.style.pointerEvents = "none";
    img.style.zIndex = "5";
    img.classList.add('sign-ink-save')

    let documentContainer = document.querySelector('.document-container')

    const rect = this.rect

    documentContainer.addEventListener("mousemove", function(event) {
      // 获取光标的位置
      var x = event.clientX;
      var y = event.clientY;

      let offsetX = rect.width / 2
      let offsetY = rect.height / 2

      // 设置图片的位置
      img.style.left = (x - offsetX) + "px";
      img.style.top = (y - offsetY) + "px";
    });

    this.img = img;

    // 添加图片到页面
    documentContainer.appendChild(this.img);
    this.documentContainer = documentContainer
  }
  
  getData (layerData) {
    if (this.path.length === 0) return
    this.layerData = layerData;
    // const annotation = {
    //   operate: "add-annot",
    //   type: 'ink',
    //   page: this.layerData.page + 1,
    //   lineWidth: 2,
    //   transparency: 0.8,
    //   borderColor: '#000000',
    //   arrow: false,
    // }
    const annotation = {
      operate: "add-annot",
      type: "image",
      page: this.layerData.page + 1,
      imageBase64: this.dataURL,
      base64content: true,
      // targetPage: pageNum,
      // date: new Date(),
      // name: uuidv4()
    }
    const { path, rect } = this.formatPath()

    annotation.rect = rect
    // annotation.inklist = path

    this.eventBus.dispatch('createSignature', annotation)
  }

  formatPath () {
    let path = ''
    const length = this.path.length
    
    const xArray = [];
    const yArray = [];

    for (let i = 0; i < length; i = i + 2) {
      const point = this.handlePoint(this.path[i])

      xArray.push(point.x);
      yArray.push(point.y);

      path+=(point.x.toFixed(2) + ',' + point.y.toFixed(2))
      if (i + 1 < length) {

        const point = this.handlePoint(this.path[i])

        xArray.push(point.x);
        yArray.push(point.y);

        path+=(';' + point.x.toFixed(2) + ',' + point.y.toFixed(2))
        if (i + 2 < length) {
          path += ';'
        }
      }
    }

    const left = Math.min(...xArray);
    const bottom = Math.min(...yArray);
    const right = Math.max(...xArray);
    const top = Math.max(...yArray);

    return {
      path,
      rect: `${left},${bottom},${right},${top}`
    }
  }

  handlePoint (point) {
    const newPoint = getInitialPoint(
      {
        x: point[0] + this.layerData.clickX - this.rect.width / 2,
        y: point[1] + this.layerData.clickY - this.rect.height / 2
      },
      this.layerData.viewport,
      this.layerData.viewport.scale
    )
    return {
      x: newPoint.x,
      y: newPoint.y
    }
  }

  initCursorStyle () {
    if (this.img) {
      this.img.remove()
    }
  }

  paintFinalCanvas () {
    let finalCanvas = document.createElement("canvas")
    let ctx = finalCanvas.getContext('2d')

    finalCanvas.width = this.rect.width
    finalCanvas.height = this.rect.height

    ctx.fillStyle = 'white'
    ctx.fillRect(0, 0, finalCanvas.width, finalCanvas.height)
    ctx.lineWidth = 3;
    ctx.strokeStyle = 'black';

    console.log(this.rect)
    ctx.drawImage(this.canvas, this.rect.left, this.rect.top, finalCanvas.width, finalCanvas.height, 0, 0, finalCanvas.width, finalCanvas.height)

    return finalCanvas
  }
}

export { InkSign }
